@@ -495,6 +495,18 @@ func (a *API) apiBaseURL() string {
 	return resolveRelative(base, rel)
 }
 
+func (a *API) mtlsAPIBaseURL() string {
+	if a.doc.MTLSRootURL != "" {
+		return resolveRelative(a.doc.MTLSRootURL, a.doc.ServicePath)
+	}
+	// TODO(andyrzhao): Remove the workaround below when MTLSRootURL becomes available in
+	// compute discovery doc, after compute migrates to OP discovery doc gen (ETA 2021).
+	if a.doc.MTLSRootURL == "" && a.doc.RootURL == "https://compute.googleapis.com/" {
+		return resolveRelative("https://compute.mtls.googleapis.com/", a.doc.ServicePath)
+	}
+	return ""
+}
+
 func (a *API) needsDataWrapper() bool {
 	for _, feature := range a.doc.Features {
 		if feature == "dataWrapper" {
@@ -719,6 +731,9 @@ func (a *API) GenerateCode() ([]byte, error) {
 	pn("const apiName = %q", a.doc.Name)
 	pn("const apiVersion = %q", a.doc.Version)
 	pn("const basePath = %q", a.apiBaseURL())
+	if mtlsBase := a.mtlsAPIBaseURL(); mtlsBase != "" {
+		pn("const mtlsBasePath = %q", mtlsBase)
+	}
 
 	a.generateScopeConstants()
 	a.PopulateSchemas()
@@ -741,6 +756,9 @@ func (a *API) GenerateCode() ([]byte, error) {
 		pn("opts = append([]option.ClientOption{scopesOption}, opts...)")
 	}
 	pn("opts = append(opts, internaloption.WithDefaultEndpoint(basePath))")
+	if a.mtlsAPIBaseURL() != "" {
+		pn("opts = append(opts, internaloption.WithDefaultMTLSEndpoint(mtlsBasePath))")
+	}
 	pn("client, endpoint, err := htransport.NewClient(ctx, opts...)")
 	pn("if err != nil { return nil, err }")
 	pn("s, err := New(client)")
@@ -824,7 +842,7 @@ func (a *API) generateScopeConstants() {
 		n++
 		ident := scopeIdentifier(scope)
 		if scope.Description != "" {
-			a.p("%s", asComment("\t", scope.Description))
+			a.p("%s", asComment("\t", removeMarkdownLinks(scope.Description)))
 		}
 		a.pn("\t%s = %q", ident, scope.ID)
 	}
@@ -887,7 +905,7 @@ func (p *Property) Default() string {
 }
 
 func (p *Property) Description() string {
-	return p.p.Schema.Description
+	return removeMarkdownLinks(p.p.Schema.Description)
 }
 
 func (p *Property) Enum() ([]string, bool) {
@@ -948,6 +966,7 @@ var pointerFields = []fieldName{
 	{api: "cloudmonitoring:v2beta2", schema: "Point", field: "Int64Value"},
 	{api: "cloudmonitoring:v2beta2", schema: "Point", field: "StringValue"},
 	{api: "compute:alpha", schema: "ExternalVpnGateway", field: "Id"},
+	{api: "compute:alpha", schema: "MetadataItems", field: "Value"},
 	{api: "compute:alpha", schema: "Scheduling", field: "AutomaticRestart"},
 	{api: "compute:beta", schema: "ExternalVpnGateway", field: "Id"},
 	{api: "compute:beta", schema: "MetadataItems", field: "Value"},
@@ -1327,7 +1346,7 @@ func (s *Schema) writeVariant(api *API, v *disco.Variant) {
 }
 
 func (s *Schema) Description() string {
-	return s.typ.Description
+	return removeMarkdownLinks(s.typ.Description)
 }
 
 func (s *Schema) writeSchemaStruct(api *API) {
@@ -1363,7 +1382,12 @@ func (s *Schema) writeSchemaStruct(api *API) {
 		p.assignedGoName = pname
 		des := p.Description()
 		if des != "" {
-			s.api.p("%s", asComment("\t", fmt.Sprintf("%s: %s", pname, des)))
+			if pname == "Deprecated" {
+				// Workaround to not trip up linters on fields named Deprecated.
+				s.api.p("%s", asComment("\t", fmt.Sprintf("%s -- %s", pname, des)))
+			} else {
+				s.api.p("%s", asComment("\t", fmt.Sprintf("%s: %s", pname, des)))
+			}
 		}
 		addFieldValueComments(s.api.p, p, "\t", des != "")
 
@@ -1400,7 +1424,7 @@ func (s *Schema) writeSchemaStruct(api *API) {
 
 	commentFmtStr := "%s is a list of field names (e.g. %q) to " +
 		"unconditionally include in API requests. By default, fields " +
-		"with empty values are omitted from API requests. However, " +
+		"with empty or default values are omitted from API requests. However, " +
 		"any non-pointer, non-interface field appearing in %s will " +
 		"be sent to the server regardless of whether the field is " +
 		"empty or not. This may be used to include empty fields in " +
@@ -1753,6 +1777,12 @@ func (m *Method) OptParams() []*Param {
 	})
 }
 
+func (m *Method) ReqParams() []*Param {
+	return m.grepParams(func(p *Param) bool {
+		return p.p.Required
+	})
+}
+
 func (meth *Method) cacheResponseTypes(api *API) {
 	if retType := responseType(api, meth.m); retType != "" && strings.HasPrefix(retType, "*") {
 		api.responseTypes[retType] = true
@@ -1813,7 +1843,21 @@ func (meth *Method) generateCode() {
 	pn(" header_ http.Header")
 	pn("}")
 
-	p("\n%s", asComment("", methodName+": "+meth.m.Description))
+	p("\n%s", asComment("", methodName+": "+removeMarkdownLinks(meth.m.Description)))
+
+	// Add required parameter docs.
+	params := meth.ReqParams()
+	// Sort to the same order params are listed in method.
+	sort.Slice(params, func(i, j int) bool { return params[i].p.Name < params[j].p.Name })
+	for i, v := range params {
+		if i == 0 {
+			p("//\n")
+		}
+		des := v.p.Description
+		des = strings.Replace(des, "Required.", "", 1)
+		des = strings.TrimSpace(des)
+		p("%s", asFuncParmeterComment("", fmt.Sprintf("- %s: %s", depunct(v.p.Name, false), removeMarkdownLinks(des)), true))
+	}
 	if res != nil {
 		if url := canonicalDocsURL[fmt.Sprintf("%v%v/%v", docsLink, res.Name, meth.m.Name)]; url != "" {
 			pn("// For details, see %v", url)
@@ -1866,7 +1910,7 @@ func (meth *Method) generateCode() {
 		des := opt.p.Description
 		des = strings.Replace(des, "Optional.", "", 1)
 		des = strings.TrimSpace(des)
-		p("\n%s", asComment("", fmt.Sprintf("%s sets the optional parameter %q: %s", setter, opt.p.Name, des)))
+		p("\n%s", asComment("", fmt.Sprintf("%s sets the optional parameter %q: %s", setter, opt.p.Name, removeMarkdownLinks(des))))
 		addFieldValueComments(p, opt, "", true)
 		np := new(namePool)
 		np.Get("c") // take the receiver's name
@@ -2076,7 +2120,11 @@ func (meth *Method) generateCode() {
 		pn(`gensupport.SetOptions(c.urlParams_, opts...)`)
 		pn(`res, err := c.doRequest("media")`)
 		pn("if err != nil { return nil, err }")
-		pn("if err := googleapi.CheckMediaResponse(res); err != nil {")
+		if meth.api.Name == "storage" {
+			pn("if err := googleapi.CheckMediaResponse(res); err != nil {")
+		} else {
+			pn("if err := googleapi.CheckResponse(res); err != nil {")
+		}
 		pn("res.Body.Close()")
 		pn("return nil, err")
 		pn("}")
@@ -2311,7 +2359,7 @@ func (meth *Method) NewArguments() *arguments {
 	pnames := meth.m.ParameterOrder
 	if len(pnames) == 0 {
 		// No parameterOrder; collect required parameters and sort by name.
-		for _, reqParam := range meth.grepParams(func(p *Param) bool { return p.p.Required }) {
+		for _, reqParam := range meth.ReqParams() {
 			pnames = append(pnames, reqParam.p.Name)
 		}
 		sort.Strings(pnames)
@@ -2448,40 +2496,60 @@ func (a *arguments) String() string {
 	return buf.String()
 }
 
-var urlRE = regexp.MustCompile(`^http\S+$`)
+var urlRE = regexp.MustCompile(`^\(?http\S+$`)
 
 func asComment(pfx, c string) string {
+	return asFuncParmeterComment(pfx, c, false)
+}
+
+func asFuncParmeterComment(pfx, c string, addPadding bool) string {
 	var buf bytes.Buffer
-	const maxLen = 70
+	var maxLen = 70
+	var padding string
 	r := strings.NewReplacer(
 		"\n", "\n"+pfx+"// ",
 		"`\"", `"`,
 		"\"`", `"`,
 	)
+	lineNum := 0
 	for len(c) > 0 {
+		// Adjust padding for the second line if needed.
+		if addPadding && lineNum == 1 {
+			padding = "  "
+			maxLen = 68
+		}
 		line := c
 		if len(line) < maxLen {
-			fmt.Fprintf(&buf, "%s// %s\n", pfx, r.Replace(line))
+			fmt.Fprintf(&buf, "%s// %s%s\n", pfx, padding, r.Replace(line))
 			break
 		}
 		// Don't break URLs.
+		var si int
 		if !urlRE.MatchString(line[:maxLen]) {
 			line = line[:maxLen]
+			si = strings.LastIndex(line, " ")
+		} else {
+			si = strings.Index(line, " ")
 		}
-		si := strings.LastIndex(line, " ")
-		if nl := strings.Index(line, "\n"); nl != -1 && nl < si {
+		if nl := strings.Index(line, "\n"); nl != -1 && (nl < si || si == -1) {
 			si = nl
 		}
 		if si != -1 {
 			line = line[:si]
 		}
-		fmt.Fprintf(&buf, "%s// %s\n", pfx, r.Replace(line))
+		fmt.Fprintf(&buf, "%s// %s%s\n", pfx, padding, r.Replace(line))
 		c = c[len(line):]
 		if si != -1 {
 			c = c[1:]
 		}
+		lineNum++
 	}
-	return buf.String()
+	// Add a period at the end if there is not one.
+	str := buf.String()
+	if addPadding && len(str) > 1 && str[len(str)-2:] != ".\n" {
+		str = str[:len(str)-1] + ".\n"
+	}
+	return str
 }
 
 func simpleTypeConvert(apiType, format string) (gotype string, ok bool) {
@@ -2614,3 +2682,19 @@ func addFieldValueComments(p func(format string, args ...interface{}), field Fie
 		p("%s", l)
 	}
 }
+
+// markdownLinkRe is a non-greedy regex meant to find markdown style links. It
+// also captures the name of the link.
+var markdownLinkRe = regexp.MustCompile("([^`]|\\A)(\\[([^\\[]*?)]\\((.*?)\\))([^`]|\\z)")
+
+func removeMarkdownLinks(input string) string {
+	out := input
+	sm := markdownLinkRe.FindAllStringSubmatch(input, -1)
+	if len(sm) == 0 {
+		return out
+	}
+	for _, match := range sm {
+		out = strings.Replace(out, match[2], fmt.Sprintf("%s (%s)", match[3], match[4]), 1)
+	}
+	return out
+}
